package com.galaxis.manatee.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.dao.ChuanyunGroupProjectDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunGroupProjectDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunManagerProjectDO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.task.ChuanyunBasicDataScheduledTask;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;

@Service
@Slf4j
public class ProjectMemberService {

    private final ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;

    public ProjectMemberService(ChuanyunBasicDataScheduledTask chuanyunBasicDataScheduledTask, ChuanYunManager chuanYunManager, ChuanyunGroupProjectDao chuanyunGroupProjectDao) {
        this.chuanyunBasicDataScheduledTask = chuanyunBasicDataScheduledTask;
        this.chuanYunManager = chuanYunManager;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
    }

    @Async("threadPoolTaskExecutor")
    public void updateManagerMember(String param) throws InterruptedException {
        log.info("同步管理项目组成员");
        var objectMapper = new ObjectMapper();
        param = param.substring(14, param.length() - 2);
        log.info(param);
        AtomicInteger atomicInteger = new AtomicInteger();
        boolean updateFlag = false;
        while (!updateFlag) {
            int atomic = atomicInteger.getAndIncrement();
            if (atomic >= 20) {
                updateFlag = true;
            }
            TimeUnit.SECONDS.sleep(5);
            var chuanyunFindResponse = chuanYunManager.find(ChuanyunManagerProjectDO.SCHEMA_CODE, param);
            ChuanyunManagerProjectDO chuanyunManagerProjectDO = objectMapper.convertValue(chuanyunFindResponse.getReturnData().getBizObject(), new TypeReference<>() {
            });
            if (chuanyunManagerProjectDO != null) {
                updateFlag = true;
                try {
                    List<String> matchers = new ArrayList<>();
                    matchers.add("F0000002_2," + chuanyunManagerProjectDO.getSeqNo());
                    var filter = Filter.instance(0, 1, true, "And", matchers);
                    ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
                    if (chuanyunFindAllResponse.getReturnData() != null) {
                        List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                        });
                        log.info("集团项目" + result.get(0));
                        chuanyunBasicDataScheduledTask.chuanyunUpdateProjectMemberManually(result.get(0));
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    log.warn(e.getMessage());
                }
                log.info("管理项目更新项目组成员成功");
            }
        }
    }

    @Async("threadPoolTaskExecutor")
    public void updateSaleMember(String param) throws InterruptedException {
        log.info("同步销售项目组成员");
        var objectMapper = new ObjectMapper();
        param = param.substring(14, param.length() - 2);
        log.info(param);
        AtomicInteger atomicInteger = new AtomicInteger();
        boolean updateFlag = false;
        while (!updateFlag) {
            int atomic = atomicInteger.getAndIncrement();
            if (atomic >= 20) {
                updateFlag = true;
            }
            try {
                TimeUnit.SECONDS.sleep(5);
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000002_2," + param);
                var filter = Filter.instance(0, 1, true, "And", matchers);
                ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() != null) {
                    updateFlag = true;
                    List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                    });
                    log.info("集团项目" + result.get(0));
                    chuanyunBasicDataScheduledTask.chuanyunUpdateProjectMemberManually(result.get(0));
                    log.info("销售项目更新项目组成员成功");
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.warn(e.getMessage());
            }
        }
    }

    @Async("threadPoolTaskExecutor")
    public void updateDevelopMember(String param) throws InterruptedException {
        log.info("同步研发项目组成员");
        var objectMapper = new ObjectMapper();
        param = param.substring(14, param.length() - 2);
        log.info(param);
        boolean updateFlag = false;
        AtomicInteger atomicInteger = new AtomicInteger();
        while (!updateFlag) {
            int atomic = atomicInteger.getAndIncrement();
            if (atomic >= 20) {
                updateFlag = true;
            }
            try {
                TimeUnit.SECONDS.sleep(5);
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000002_2," + param);
                var filter = Filter.instance(0, 1, true, "And", matchers);
                ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() != null) {
                    updateFlag = true;
                    List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                    });
                    log.info("集团项目" + result.get(0));
                    chuanyunBasicDataScheduledTask.chuanyunUpdateProjectMemberManually(result.get(0));
                    log.info("研发项目更新项目组成员成功");
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.warn(e.getMessage());
            }
        }
    }

    @Async("threadPoolTaskExecutor")
    public void updateDeliveryMember(String param) throws InterruptedException {
        log.info("同步交付类项目组成员");
        var objectMapper = new ObjectMapper();
        param = param.substring(14, param.length() - 2);
        log.info(param);
        boolean updateFlag = false;
        AtomicInteger atomicInteger = new AtomicInteger();
        while (!updateFlag) {
            int atomic = atomicInteger.getAndIncrement();
            if (atomic >= 20) {
                updateFlag = true;
            }
            try {
                TimeUnit.SECONDS.sleep(5);
                List<String> matchers = new ArrayList<>();
                matchers.add("F0000002_2," + param);
                var filter = Filter.instance(0, 1, true, "And", matchers);
                ChuanyunFindAllBizDTO<Object> chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() != null) {
                    updateFlag = true;
                    List<ChuanyunGroupProjectDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                    });
                    result.forEach(chuanyunGroupProjectDO -> {
                        log.info("集团项目: {}", chuanyunGroupProjectDO);
                        chuanyunBasicDataScheduledTask.chuanyunUpdateProjectMemberManually(chuanyunGroupProjectDO);
                        log.info("交付类项目更新项目组成员成功");
                    });
                }
            } catch (Exception e) {
                e.printStackTrace();
                log.warn(e.getMessage());
            }
        }
    }
}
