package com.galaxis.manatee.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindAllBizReturnData;
import com.galaxis.manatee.entity.chuanyun.dto.ChuanyunFindBizDTO;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.manager.ChuanYunManager;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import java.math.BigDecimal;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@SpringBootTest
public class ProjectImplTest {

    @Autowired
    private ChuanYunManager chuanYunManager;

    @Test
    public void saveGroupProject() throws Exception {
        List<String> m1 = List.of("F0000075_5,异常");
        Filter instance = Filter.instance(0, 50, true, Filter.AND, m1);
        ChuanyunFindAllBizDTO<Object> objectChuanyunFindBizDTO = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, instance);

        // 拿到所有项目
        ChuanyunFindAllBizReturnData<Object> returnData = objectChuanyunFindBizDTO.getReturnData();
        List<Object> bizObjectArray = returnData.getBizObjectArray();
        ObjectMapper objectMapper = new ObjectMapper();
        List<ChuanyunGroupProjectDO> groupProjectDOList = objectMapper.convertValue(bizObjectArray, new TypeReference<>() {
        });
        List<String> disProjectCodeList = groupProjectDOList.stream().map(ChuanyunGroupProjectDO::getProjectCode).toList().stream().distinct().toList();
        disProjectCodeList.forEach(code -> {
            List<String> m2 = List.of("F0000003_2," + code);
            Filter f1 = Filter.instance(0, 10, true, Filter.AND, m2);
            ChuanyunFindAllBizDTO<ChuanyunGroupProjectDO> buildResponse = null;
            try {
                buildResponse = chuanYunManager.findAll(ChuanyunBuildProject.SCHEMA_CODE, f1);
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
            if (buildResponse.getReturnData() == null) {
                return;
            }
            List<ChuanyunBuildProject> result = objectMapper.convertValue(buildResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
            });
            List<String> buildProjectIdList = result.stream().map(ChuanyunBuildProject::getObjectId).toList();
            buildProjectIdList.forEach(buildProjectId -> {
                List<String> m3 = List.of("F0000001_2," + buildProjectId);
                Filter filter = Filter.instance(0, 10, true, Filter.AND, m3);
                ChuanyunFindAllBizDTO<Object> projectList = null;
                try {
                    projectList = chuanYunManager.findAll(ChuanyunProjectOrganizationStructureDO.SCHEMA_CODE, filter);
                } catch (JsonProcessingException e) {
                    e.printStackTrace();
                }
                if (projectList == null || projectList.getReturnData() == null) {
                    return;
                }
                List<ChuanyunProjectOrganizationStructureDO> projectOrganizationStructureList = objectMapper.convertValue(projectList.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                List<String> parentIdList = projectOrganizationStructureList.stream().map(BasicDO::getObjectId).toList();
                parentIdList.forEach(parentId -> {
                    ChuanyunFindBizDTO<Object> workGroupResponse = chuanYunManager.find(ChuanyunWorkGroupDO.SCHEMA_CODE, parentId);
                    if (workGroupResponse == null || !workGroupResponse.getSuccessful()) {
                        return;
                    }
                    if (workGroupResponse.getReturnData() == null) {
                        return;
                    }
                    List<ChuanyunWorkGroupDO> workGroupList = objectMapper.convertValue(workGroupResponse.getReturnData().getBizObject(), new TypeReference<>() {
                    });
                    workGroupList.forEach(chuanyunWorkGroupDO -> {
                        BigDecimal total = new BigDecimal("0.00");
                        Map<String, BigDecimal> workGroupAmount = new HashMap<>();
                        List<String> userIdList = chuanyunWorkGroupDO.getSubWorkGroupMemberObject().stream().map(BasicSubDO::getObjectId).toList();
                        // 查询报销单

                        for (String userId : userIdList) {
                            // 查询用户在该项目的报销单
                            Filter f = Filter.instance(0, 10, true, Filter.AND, List.of("F0000093_2," + buildProjectId, "F0000001_2," + userId));
                            ChuanyunFindAllBizDTO<Object> reimbursementResponse = null;
                            try {
                                reimbursementResponse = chuanYunManager.findAll(ChuanyunReimbursementDO.SCHEMA_CODE, f);
                            } catch (JsonProcessingException e) {
                                e.printStackTrace();
                            }
                            List<ChuanyunReimbursementDO> reimbursementDOList = objectMapper.convertValue(reimbursementResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            BigDecimal totalReim = reimbursementDOList.stream().map(ChuanyunReimbursementDO::getTotalAmountNotTaxed).reduce(BigDecimal.ZERO, BigDecimal::add);
                            total = total.add(totalReim);
                        }
                        // 统计工作组所有的工时
                        workGroupAmount.put(chuanyunWorkGroupDO.getObjectId(), total);
                        System.out.println(workGroupAmount);
                    });
                });
            });
        });

    }

    @Test
    public void buildProjectTest() {
        ObjectMapper objectMapper = new ObjectMapper();
        List<String> m2 = List.of("F0000003_2,D20084");
        Filter f1 = Filter.instance(0, 10, true, Filter.AND, m2);
        ChuanyunFindAllBizDTO<ChuanyunGroupProjectDO> buildResponse = null;
        try {
            buildResponse = chuanYunManager.findAll(ChuanyunBuildProject.SCHEMA_CODE, f1);
        } catch (JsonProcessingException e) {
            e.printStackTrace();
        }
        if (buildResponse.getReturnData() == null) {
            return;
        }
        List<ChuanyunBuildProject> result = objectMapper.convertValue(buildResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
        });
        List<String> buildProjectIdList = result.stream().map(ChuanyunBuildProject::getObjectId).toList();
        buildProjectIdList.forEach(buildProjectId -> {
            List<String> m3 = List.of("F0000001_2," + buildProjectId);
            Filter filter = Filter.instance(0, 10, true, Filter.AND, m3);
            ChuanyunFindAllBizDTO<Object> projectList = null;
            try {
                projectList = chuanYunManager.findAll(ChuanyunProjectOrganizationStructureDO.SCHEMA_CODE, filter);
            } catch (JsonProcessingException e) {
                e.printStackTrace();
            }
            if (projectList == null || projectList.getReturnData() == null) {
                return;
            }
            List<ChuanyunProjectOrganizationStructureDO> projectOrganizationStructureList = objectMapper.convertValue(projectList.getReturnData().getBizObjectArray(), new TypeReference<>() {
            });
            List<String> parentIdList = projectOrganizationStructureList.stream().map(BasicDO::getObjectId).toList();
            parentIdList.forEach(parentId -> {
                List<String> m4 = List.of("parentObjectId_2," + parentId);
                ChuanyunFindAllBizDTO<Object> workGroupResponse = null;
                try {
                    workGroupResponse = chuanYunManager.findAll(ChuanyunWorkGroupDO.SCHEMA_CODE, Filter.instance(0, 30, true, Filter.AND, m4));
                } catch (JsonProcessingException e) {
                    e.printStackTrace();
                }
                if (workGroupResponse == null || !workGroupResponse.getSuccessful()) {
                    return;
                }
                if (workGroupResponse.getReturnData() == null) {
                    return;
                }
                List<ChuanyunWorkGroupDO> workGroupList = objectMapper.convertValue(workGroupResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                workGroupList.forEach(chuanyunWorkGroupDO -> {

                    Map<String, BigDecimal> workGroupAmount = new HashMap<>();
                    List<String> userIdList = chuanyunWorkGroupDO.getSubWorkGroupMemberObject().stream().map(BasicSubDO::getObjectId).toList();
                    // 查询报销单
                    BigDecimal total = new BigDecimal("0.00");
                    for (String userId : userIdList) {
                        // 查询用户在该项目的报销单
                        List<String> m1 = List.of("F0000075_5,异常", "F0000002_2,D20084");
                        Filter instance = Filter.instance(0, 50, true, Filter.AND, m1);
                        ChuanyunFindAllBizDTO<Object> groupProjectResponse = null;
                        try {
                            groupProjectResponse = chuanYunManager.findAll(ChuanyunGroupProjectDO.SCHEMA_CODE, instance);
                        } catch (JsonProcessingException e) {
                            e.printStackTrace();
                        }
                        if (!groupProjectResponse.getSuccessful() || groupProjectResponse.getReturnData() == null) {
                            continue;
                        }
                        List<Object> bizObjectArray = groupProjectResponse.getReturnData().getBizObjectArray();
                        List<ChuanyunGroupProjectDO> groupProjectDOList = objectMapper.convertValue(bizObjectArray, new TypeReference<>() {
                        });

                        for (ChuanyunGroupProjectDO chuanyunGroupProjectDO : groupProjectDOList) {
                            Filter f = Filter.instance(0, 10, true, Filter.AND, List.of("F0000093_2," + chuanyunGroupProjectDO.getObjectId(), "F0000001_2," + userId));
                            ChuanyunFindAllBizDTO<Object> reimbursementResponse = null;
                            try {
                                reimbursementResponse = chuanYunManager.findAll(ChuanyunReimbursementDO.SCHEMA_CODE, f);
                            } catch (JsonProcessingException e) {
                                e.printStackTrace();
                            }
                            if (!reimbursementResponse.getSuccessful() || reimbursementResponse.getReturnData() == null) {
                                continue;
                            }
                            List<ChuanyunReimbursementDO> reimbursementDOList = objectMapper.convertValue(reimbursementResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                            });
                            BigDecimal totalReim = reimbursementDOList.stream().map(ChuanyunReimbursementDO::getTotalAmountNotTaxed).reduce(BigDecimal.ZERO, BigDecimal::add);
                            total = total.add(totalReim);
                        }
                    }
                    // 统计工作组所有的工时
                    workGroupAmount.put(chuanyunWorkGroupDO.getObjectId(), total);
                    System.out.println(workGroupAmount);
                });
            });
        });


    }

}
